package model

import (
	"fmt"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/error_process"
	bgm "git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/util"
	"github.com/pkg/errors"
	"github.com/spf13/cast"
	"gorm.io/gorm"
)

type (
	Category struct {
		Value string `json:"value" gorm:"column:value"`
		Uuid  string `json:"uuid" gorm:"primary_key" valid:"no_empty"`
		IsDel int    `json:"-" gorm:"column:is_del"`
		Tags  *[]Tag `json:"tags,omitempty" gorm:"many2many:c_tag;foreignKey:Uuid;joinForeignKey:CategoryUid;References:Uuid;JoinReferences:TagUid" valid:"no_empty"`
	}
	CategoriesTagDto struct {
		TIds []string `json:"tag_ids" valid:"no_empty"`
		Cid  string   `json:"category_id" valid:"no_empty"`
	}
)

func (m *Category) TableName() string {
	return "category"
}

func GetCategoryM(page, size int, query bgm.QueryMap, db *gorm.DB) ([]Category, int64, error) {
	i := new([]Category)
	num := int64(0)
	for k, v := range query {
		db = db.Where(fmt.Sprintf("%v = ?", k), v)
	}
	if num = db.Find(i).RowsAffected; num <= 0 {
		return nil, 0, error_process.GetErr(errors.New(error_process.GormError), query)
	}
	if size > 0 {
		db = db.Limit(size).Offset((page - 1) * size)
	}
	if err := db.Preload("Tags").Find(i).Error; err != nil {
		return nil, 0, err
	}
	return *i, num, nil
}

func CreateCategoryM(category *Category, db *gorm.DB) error {
	if err := db.Create(category).Error; err != nil {
		return error_process.CreateErr(err, category)
	}
	return nil
}

func UpdateCategoryM(category *Category, db *gorm.DB) error {
	if num := db.Model(Category{}).Where("uuid = ?", category.Uuid).Updates(*category).RowsAffected; num != 1 {
		return error_process.UpdateErr(category, category.Uuid)
	}
	return nil
}

func CreateTagForCategoryM(cid string, tIds []string, db *gorm.DB) error {
	c := new(Category)
	//step1 getCategory
	if err := db.Where(&Category{
		Uuid: cid,
	}).Find(c).Error; err != nil {
		return error_process.GetErr(err, c)
	}
	//step2 get tags and check num is expected
	var ts []Tag
	if num := db.Where(tIds).Find(&ts).RowsAffected; num != cast.ToInt64(len(tIds)) {
		return error_process.RecordNotMatchWithUniqueErr(tIds, len(tIds), cast.ToInt(num))
	}
	//step3 add association
	if err := db.Model(c).Association("Tags").Append(ts); err != nil {
		return error_process.CommonErr(err)
	}
	return nil
}

func DelTagForCategoryM(cid string, tIds []string, db *gorm.DB) error {
	c := new(Category)
	//step1 getCategory
	if err := db.Where(&Category{
		Uuid: cid,
	}).Find(c).Error; err != nil {
		return error_process.GetErr(err, c)
	}
	//step2 get tags and check num is expected
	var ts []Tag
	if num := db.Where(tIds).Find(&ts).RowsAffected; num != cast.ToInt64(len(tIds)) {
		return error_process.RecordNotMatchWithUniqueErr(tIds, len(tIds), cast.ToInt(num))
	}
	//step3 add association
	if err := db.Model(c).Association("Tags").Delete(ts); err != nil {
		return error_process.CommonErr(err)
	}
	return nil
}
