package service

import (
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/error_process"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/gorm_helper"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/model"
	bgm "git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/util"
	"github.com/gofiber/fiber/v2"
	"gorm.io/gorm"
)

func CreateCategory(category *model.Category, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.CreateCategoryM(category, db)
}

func GetCategory(page, limit int, query bgm.QueryMap, ctx *fiber.Ctx) ([]model.Category, int64, error) {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return nil, 0, err
	}
	return model.GetCategoryM(page, limit, query, db)
}

func UpdateCategory(category *model.Category, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.UpdateCategoryM(category, db)
}

func DeleteCategory(category *model.Category, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	category.IsDel = 1
	//根据uuid对 is_del字段update，其他字段不允许传入
	if category.Value != "" {
		return error_process.ExtraValueErr("Value")
	}
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.UpdateCategoryM(category, db)
}

// CreateTagForCategory 为Category 新增tag
func CreateTagForCategory(cTags *model.CategoriesTagDto, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.CreateTagForCategoryM(cTags.Cid, cTags.TIds, db)
}

// DeleteTagForCategory 删除Category分组
func DeleteTagForCategory(cTags *model.CategoriesTagDto, ctx *fiber.Ctx) error {
	var (
		db  *gorm.DB
		err error
	)
	if db, err = gorm_helper.GetContextDB(ctx); err != nil {
		return err
	}
	return model.DelTagForCategoryM(cTags.Cid, cTags.TIds, db)
}
